# Released under the GNU General Public License, v3.0 or later
# Copyright (c) 2007 Phil Jones <interstar@gmail.com>


# OPML to website

from opml import *
from os import removedirs, mkdir, rmdir, getcwd, makedirs
from shutil import copyfile, copytree

import re
from string import Template
from sys import *

import csv
from UniCsv import UnicodeReader

from gwNode import GWNode, OpmlToTree
from SymbolTable import *
from logger import *
from gwHelpers import *



# classes        
        
class ArgBlock(dict) :

    """
    when a node is interpreted as an ArgBlock we find all the ways it contains arguments that
    can be passed to a block
    - lineArgs are ,, separated args on the initial line of the call and are mapped to $_0, $_1 etc.
    - namedChildren are the children defined with
      name : val
      and are mapped to $name 
    - anonChildren are child nodes without :
      and are mapped to $,0 , $,1 , $,2   etc.
    There are also three lists : lineArgs, namedChildren, anonChildren so we can iterate through these groups
    """


    def setLineArgs(self, node) :
        
        if re.match('\S+\s+(.+)',node.text) :
            # something on the command line, at least one arg
            s = (re.match('\S+\s+(.+)',node.text)).groups()[0]
           
            if s.find(',,') < 0 :
                # only one arg,
                self.lineArgs = [s.strip()]
                self['_0'] = s.strip()
            else :
                self.lineArgs = [x.strip() for x in s.split(',,')]
                for x in range(len(self.lineArgs)) :
                    self['_%s'%x] = self.lineArgs[x]


    def setChildArgs(self, node, evalSymbols, joinChildren) :
        
        for child in node.children :
            t = evalSymbols(child.text)
            if t.find(':') > 0 :
                # now we *always* use : for named args. And args include right of line + children
                parts = t.split(':')
                val = parts[1].strip() + joinChildren(child)
                
                self.namedChildren.append(val)
                self[parts[0].strip()] = val
            else :
                # this is going to be an anon arg
                val = t.strip() + joinChildren(child)
                self.anonChildren.append(val)
                
        for x in range(len(self.anonChildren)) :
            self['__%s'%x] = self.anonChildren[x]

    
    def __init__(self, node, evalSymbols, joinChildren = lambda node : ''.join([x.text for x in node.children]),log=Logger()) :
        self.lineArgs = []
        self.namedChildren = []
        self.anonChildren = []
        self.log = log
        self.setLineArgs(node)
        self.setChildArgs(node, evalSymbols, joinChildren)
        
                 

class UrMode :

    """
    A Mode is something within which we interpret the syntax of a Node.
    Different modes *can* interpret the same syntax in different ways (although it's generally not encouraged)
    The most obvious (and useful) difference is between SiteMode and various in-page modes
    In SiteMode, nodes without any further syntax get turned into subdirectories in the file system. In
    in-page modes they are text that goes into the page.
    The UrMode class is the parent of all other modes. It's EvalNode routine supports the fundamental stuff
    in GeekWeaver like defining and calling re-usable blocks, commenting, and calling into Python.
    You really shouldn't over-ride this in a mode subclass unless you want to change this fundamental behaviour

    Instead use modeEvalNode for the stuff which is specific to your mode if you define a new one.    
    """

    def __init__(self, environment) :
        self.environment = environment

    def log(self,s, typ='normal') :
        self.environment.interpreter.log(s,typ)

    def getFromSymbolTable(self, sym) :
        return self.environment.symbolTable.get(sym)
    
    def pushFrameToSymbolTable(self, frame) :
        self.environment.symbolTable.pushFrame(frame)

    def popFrameFromSymbolTable(self) :
        self.environment.symbolTable.popFrame()

    def getMode(self,s) :
        return self.environment.interpreter.modes[s]

    def argBlock(self, node, evalSymbols = lambda x : x) :
        return ArgBlock(node, evalSymbols)

    def innerCall(self, c, argBlock, depth, cDir, sfx) :
        self.pushFrameToSymbolTable(argBlock)
        self.log('Pushed to SymbolTable')
        self.log(self.environment.symbolTable,'symTable')
        
        if c.__class__ == GWNode :
            # if what was in the symbol is a node, we just evaluate its children    
            rVal = ''.join([self.evalNode(x,depth,cDir,sfx) for x in c.children])
                    
        else :
            # what was in the symbol is a string, so we want to 
            # allow *our* children to fill in any templates 
            tpl = Template(self.evalSymbols(c))
            rVal = tpl.safe_substitute(argBlock)

        self.popFrameFromSymbolTable()
        self.log('Pop From SymbolTable')
        self.log(self.environment.symbolTable,'symTable')

        self.log('return value (rVal) is %s' % rVal)
        return rVal
    
    def callBlock(self,node, s, depth, cDir, sfx, args=[]) :
        joinChildren = jFact(self.evalNode, depth, cDir)

        # we're dereferencing a symbol, so get it from the symbol table *before* putting new args in
        c = self.getFromSymbolTable(s)

        # put args of this call (ie. children of the tree node) into the symbolTable
        blocks = ArgBlock(node,self.evalSymbols,joinChildren,self.environment.logger)

        self.log("callBlock %s (line args : %s)" % (s,args))        
        
        return self.innerCall(c,blocks, depth,cDir,sfx)
                       

    def evalNode(self, node, depth = 0, cDir = '', sideFx = lambda a,b,c : (a,b,c)) :
        joinChildren = jFact(self.evalNode, depth, cDir)
        self.log('evalNode %s ' % (node.text))
        sideFx(node, depth, cDir)
        if node.__class__ == GWNode :
            # the node is a sub-tree
            s = self.evalSymbols(node.text)
          
            # General Structural Syntax
            if re.match('::',s) :
                # we're defining a block
                s = s[2:]
                self.log('defining block : %s'%s)
                if s.find('@') >= 0 :
                    # block from template file
                    xs = [x.strip() for x in s.split('@')]
                    self.log('block defined from file : %s' % xs[1])
                    b = fromFile(xs[1])
                    self.pushFrameToSymbolTable({xs[0] : b})
                elif s.find('<') >= 0 :
                    # load a subtree from another opml file
                    xs = [x.strip() for x in s.split('<')]
                    self.log('block defined from subtree in file : %s' % xs[1])
                    self.pushFrameToSymbolTable({xs[0] : self.environment.interpreter.opmlFileToTree(xs[1])})
                    
                else :
                    # we're defining a subtree
                    # put it into the table, no cute lexical closures yet
                    self.pushFrameToSymbolTable({s.strip() : node})
                return ''                    

            if re.match('&&',s) :
                modeName = s[2:].strip()
                self.log('Mode transition to %s' % modeName)
                m = self.environment.interpreter.modes[modeName]
                blocks = [m.evalNode(x, depth, cDir, sideFx) for x in node.children]
                return ''.join(blocks)

            if re.match(':forline',s) :
                blocks = []
                blockName = s[8:].strip()
                self.log('LineArg iterator (applying block : %s to all line args)' % s)
                a = self.environment.getSymbolTable().getCurrentLineArgs()
                for x in a :
                    self.environment.getSymbolTable().pushFrame( {'_0' : x} )
                    c = self.getFromSymbolTable(blockName)
                    blocks.append(self.innerCall(c,{},depth,cDir,sideFx))
                    self.environment.getSymbolTable().popFrame()
                return ''.join(blocks)

            if re.match(':forchild',s) :
                blocks = []
                blockName = s[9:].strip()
                self.log('AnonChild iterator (applying block : %s to all anon children)' % s)
                a = self.environment.getSymbolTable().getCurrentAnonChildren()
                for x in a :
                    self.environment.getSymbolTable().pushFrame( {'_0' : x} )
                    c = self.getFromSymbolTable(blockName)
                    blocks.append(self.innerCall(c,{},depth,cDir,sideFx))
                    self.environment.getSymbolTable().popFrame()
                return ''.join(blocks)

            
            if re.match(':\*',s) :
                # call a block on each item of the sublist (kind of a comprehension)
                blocks = []
                blockName = s[2:].strip()
                self.log('List Comprehension (applying block : %s to all items in list)' % s)
                for x in node.children :
                    args = re.split('\s',x.text)
                    blocks.append(self.callBlock(x,blockName,depth,cDir, sideFx, args))
                return ''.join(blocks)

            if re.match(':',s) :
                self.log('Block call : %s' % s)
                ll = LexLine(s,':')
                return self.callBlock(node,ll.key,depth,cDir, sideFx, ll.args)
            
            if re.match('!',s) :
                # it's a python call (another dangerous thing)
                return eval(s[1:])                

            if re.match('//',s) :
                # this node and sub-tree commented out
                return ''
            
            # If we got here then the line was neither defining a block, calling a block, nor calling a bit of python
            return self.modeEvalNode(node, depth, cDir, sideFx)

        
        else :
            # the nood is actually a string so we just evaluate any symbols in it and return it
            return self.evalSymbols(node)

    def modeEvalNode(self, node, depth = 0, cDir = '', sideFx = lambda x : x) :
        # this method likely to be over-ridden in the sub-classes
        # right now, because we don't have more specific information
        # we'll just create a string from the text and evaluating the children
        s = self.evalSymbols(node.text)
        return s + ''.join([self.evalNode(x,depth,cDir,sideFx) for x in node.children]) 

    def evalNonArgChildren(self, node, joinChildren) :
        blocks = {}
        for child in node.children :
            blocks[child.text.strip()] = joinChildren(child)
        return blocks

    def symbolAsString(self, sym) :
        v = self.getFromSymbolTable(sym)
        if v.__class__ == GWNode :
            return ''.join([self.evalNode(x) for x in v.children])
        else :
            return v

        
    def evalSymbols(self, s) :
        self.log('evalSymbols with %s' % s)
        
        if s.find('$') >= 0 :

            d = {} # dictionary into which we'll put all symbols

            if s.find('$/') >= 0 :
                # this handles the vars with default values
                r = re.compile(r'\$\/(?P<name>[A-Za-z0-9_]+)\/(?P<default>\S*)\/')
                ds = r.findall(s)
                for m in ds :
                    try :
                        vm = self.symbolAsString(m[0])
                        s = r.sub('$\g<name>',s)
                        d[m[0]] = vm
                    except :
                        s = r.sub('\g<default>',s)
                
            r = re.compile(r'\$([A-Za-z0-9_]+)')
            ms = r.findall(s)
            r = re.compile(r'\$\{([A-Za-z0-9_\s]+)\}')
            ms = ms + r.findall(s)
            for m in ms :
                try :
                    d[m] = self.symbolAsString(m)
                except : pass
            return Template(s).safe_substitute(d)
        else :
            return s

    
class PrimalMode(UrMode) :

    """
    This is the mode that we start in, outside SiteMode
    """
    
    def modeEvalNode(self, node, depth=0, cDir = '', sideFx = lambda x : x) :
        
        s = self.evalSymbols(node.text)  
        self.log('evalPrimalNode %s' % s)
        if re.match('@index',s) :
            # define template file
            self.log('evalPrimalNode\defining index template')
            fName = s[6:].strip()
            self.environment.interpreter.indexTemplate = Template(fromFile(fName))

        elif re.match('@fileExt',s) :
            # define default output file extension
            self.log('evalPrimalNode\defining default file extension')
            ext = s[8:].strip()
            self.environment.interpreter.defaultFileExtension = ext

        elif re.match('@root',s) :
            # define default output file extension
            self.log('evalPrimalNode\defining (coded) siteRoot')
            r = s[5:].strip()
            self.environment.interpreter.codedSiteRoot = r
            
        elif s[0] == '&' :
            # from primal, this is starting the site
            self.log('starting site')
            self.log('make siteMap')
            self.log('cDir is %s' % cDir)
            self.log('root is %s' % self.environment.siteRoot())
            self.environment.siteMap = self.environment.interpreter.makeSiteMap(node, cDir)
            self.log('finish starting a site')
            return self.getMode('staticSite').evalNode(node, depth+1, self.environment.interpreter.dName, sideFx)

        return ''.join([self.evalNode(x,depth+1,cDir,sideFx) for x in node.children])


        
class StaticSiteMode(UrMode) :

    def modeEvalNode(self, node, depth, cDir, sideFx) :
        s = self.evalSymbols(node.text)  
        self.log('evalSiteNode : %s ' % s)
        nd = cDir
            
        if '>'.find(s[0]) > -1 :
            self.log('evalSiteNode\doing nothing with %s' % s[0])

        elif re.match('@cDir',s) :
            nd = cDir + '/' + spaceUnder(s[5:].strip())
            self.log('evalSiteNode\changing the cDir to %s' % nd)

        else :
            if s[0] == '&'  : s = s[1:]
            nd = cDir + '/' + spaceUnder(s)
            self.log('evalSiteNode\creating directory %s (cDir %s)' % (nd, cDir))
            makedirs(nd)
            self.log('evalSiteNode\adding to siteMap %s ' % nd)
                     
        index = []
        self.log('evalSiteNode\starting index loop')
        for x in node.children :
            self
            if x.text == '' :
                pass
            elif x.text[0] == '&' :
                lpn = self.processPage(x, depth+1, nd, sideFx)
                i = [lpn.pageName,spaceUnder(lpn.outFileName),False]
                index.append(i)
                self.log('index += %s'%i)

            elif x.text[0] == ':' :
                self.log('eval block from StaticSiteMode')
                self.evalNode(x, depth, cDir, sideFx)
            
            elif re.match('>\s*(.+)>\s*(\S+)',x.text) :
                m = re.match('>\s*(.+)\s*>\s*(\S+)',x.text)
                linkText = (m.groups(0)[0]).strip()
                dest = m.groups(0)[1]
                index.append([linkText,dest,False])

            else :
                self.evalNode(x, depth+1, nd, sideFx)
                index.append([x.text,spaceUnder(x.text)+'/index.html',True])                    

        self.environment.interpreter.indexPage(node.text, index, nd)

        return ''

    def processPage(self, node, depth, cDir, sideFx) :
        # now we're inside a page beause we hit a &
        sideFx(node,depth,cDir)
        paras = []
        text = self.evalSymbols(node.text)
        self.log('processPage : node.text is %s ' % text)
        lpn = LexPageName(text,self.environment.interpreter.defaultFileExtension)
        
        if lpn.matches == True :
            self.environment.currentPageName = lpn.pageName
            if lpn.symbol != '' :
                page = self.getMode('html').callBlock(node, lpn.symbol, depth, cDir, sideFx)                             
            else :
                paras = [self.getMode('html').evalNode(x, depth, cDir, sideFx) for x in node.children ]
                page = ''.join(paras)

                
        self.environment.siteMapper.add(cDir,spaceUnder(lpn.outFileName),spaceUnder(lpn.outFileName),page.encode('utf-8'))

        return lpn


class HtmlMode(UrMode) :

    def patternReplace(self,t) :
        if re.search(r'\[\[(\S+)\s*(\S+)\]\]',t) :
            t = re.sub(r'(\[\[(?P<url>\S+)\s*(?P<text>\S+)\]\])',"<a href='" + self.environment.siteRoot() + "\g<url>'>\g<text></a>",t)
        return t


    def modeEvalNode(self, node, depth = 0, cDir = '', sideFx = lambda x : x) :
        b = []
        t = self.evalSymbols(node.text)
        joinChildren = jFact(self.evalNode, depth, cDir, sideFx)

        t = self.patternReplace(t)
        
        if len(t) > 0 :
            
            if t[0] == '=' :
                b.append('<h2>%s</h2>' % (t[1:]).strip())
                b.append( joinChildren(node))

            elif re.match('\.<(.+)>',t) :
                m = re.match('\.<(.+)>',t)        
                b.append("""<%s>"""  % m.groups()[0] )
                b.append( joinChildren(node))

                i = m.groups()[0].find(' ')
                if i < 0 :
                    b.append('</%s>' % m.groups()[0] )
                else :
                    b.append('</%s>' % m.groups()[0][0:i] )


            elif t[0] == '.' :
                ll = LexLine(t,'.')
                b.append("""<div class='%s'>%s"""  % (ll.key,' '.join(ll.args)))
                b.append( joinChildren(node))
                b.append('</div>')

            elif t[0] == ';' :
                ll= LexLine(t,';')
                b.append("""<div id='%s'>%s"""  % (ll.key,' '.join(ll.args)))
                b.append( joinChildren(node))
                b.append('</div>')


            elif re.match('#@',t) :
                ll = LexLine(t,"#@")
                if ll.args != [] :
                    delim = ll.args[0].strip()
                else :
                    delim = ','                  
                
                self.log('opening external CSV file %s and delim is *%s* ' % (ll.key,delim))
                reader = UnicodeReader(open(ll.key, "rb"),csv.excel,'unicode_escape',delimiter=delim.encode('utf-8'))
                b.append(u"<table class='csvtable'>")
                for row in reader :
                    self.log('row : %s' % row)
                    b.append(u'<tr>%s</tr>' % ''.join([u'<td>%s</td>' % c for c in row]))
                b.append(u"</table>")

            elif t[0] == '#' :
                ll = LexLine(t,'#')
                tid = ''
                if ll.id != '' :
                    tid = " id='%s'" % ll.id
                b.append("""<table class='%s'%s>""" % (ll.cls, tid))
                if ll.args != [] :
                    b.append('\n<tr>%s</tr>' % ''.join(['<th>%s</th>' % th for th in ll.args]))                        
                        
                for row in node.children :
                    b.append("\n<tr>")
                    rText = row.text
                    if rText.find(',,') >= 0 :
                        cells = [c.strip() for c in rText.split(',,')]
                        for col in cells :
                            b.append("<td>%s</td>" % self.evalSymbols(col))
                    else :
                        for col in row.children :
                            b.append("<td>%s</td>" % self.evalNode(col,depth,cDir,sideFx))
                    b.append("</tr>")
                b.append("\n</table>")

            elif t[0] == '@' :
                tpl = Template(fromFile(t[1:]))
                blocks = self.evalArgChildren(node,joinChildren)
                b.append(tpl.safe_substitute(blocks))


            elif re.match(r'\?\s*([A-Za-z]+)\s*->\s(\S+)',t) :
                m = re.match(r'\?\s*([A-Za-z]+)\s*->\s(\S+)',t)
                b.append("<form name='%s' method='GET' action='%s'>" % (m.groups(0)[0],m.groups(0)[1]))
                for child in node.children :
                    b.append(self.getMode('htmlForm').evalNode(child, depth, cDir, sideFx))
                b.append("\n</form>")

            elif re.match(r'\?\s*([A-Za-z]+)\s*=>\s(\S+)',t) :
                m = re.match(r'\?\s*([A-Za-z]+)\s*=>\s(\S+)',t)
                b.append("<form name='%s' method='POST' action='%s'>" % (m.groups(0)[0],m.groups(0)[1]))
                for child in node.children :
                    b.append(self.getMode('htmlForm').evalNode(child, depth, cDir, sideFx))
                b.append("\n</form>")

            elif re.match(r'>\s*(.+)\s*>\s*((\S+))',t) :
                m = re.match(r'>\s*(.+)\s*>\s*((\S+))',t)
                b.append("<a href='%s'>%s</a>" % (m.groups(0)[1],m.groups(0)[0].strip()))

            elif t[0] == ']':
                ll = LexLine(t,']')
                b.append("<img src='%s' alt='%s'/>" % (ll.key,' '.join(ll.args)))

            elif t[0] == '*' :
                b.append("""<ul class='%s'>""" % (t[1:]).strip())
                for child in node.children :
                    b.append('\n<li>%s</li>' % self.evalNode(child,depth,cDir,sideFx))
                b.append('</ul>')
                
            else :
                b.append(t)
                b.append( joinChildren(node))
        else :
            # text value of this node is empty but children may still have value
            b.append(t)
            b.append( joinChildren(node))
            
        return ''.join(b)


class HtmlFormMode(HtmlMode) :

    def modeEvalNode(self, node, depth = 0, cDir = '', sideFx = lambda a,b,c : (a,b,c)) :
        s = self.evalSymbols(node.text).strip()
        self.log('HtmlFormMode modeEvalNode : %s' % s)

        if re.match(r'\?\?',s) :
            s = s[2:]
            if s.find(':') > 0 :
                parts = s.split(':')
                return "\n%s : <input name='%s'/>"  % (parts[0],parts[1])
            else :
                return "\n<input name='%s'/>" % s
        elif re.match(r'\?\*',s) :
            b = []
            b.append("\n<select name='%s'>" % s[2:])
            for x in node.children :
                parts = [y.strip() for y in self.evalSymbols(x.text).split(':')]
                b.append("\n<option value='%s'>%s</option>" % (parts[0], parts[1]))
            b.append("\n</select>")
            return ''.join(b)
        elif re.match(r'\?_',s):
            s = s[2:]
            if s.find(':') > 0 :
                parts = s.split(':')
                return """
%s : <textarea name='%s'>
</textarea>""" % (parts[0].strip(),parts[1].strip())
            else :
                return """
<textarea name='%s'>
</textarea>""" % s.strip()
        elif re.match(r'\[\]',s) :
            return "\n<input type='submit'/>"
        elif re.match(r'\[\s*([A-Za-z ]+)\s*\]',s) :
            m = re.match(r'\[\s*([A-Za-z ]+)\s*\]',s)
            return "\n<input value='%s' type='submit'/>" % m.groups(0)
        
        elif re.match(r'\[\s*([A-Za-z]+)\s*;\s*([A-Za-z]+)\s*\]',s) :
            m = re.match(r'\[\s*([A-Za-z]+)\s*;\s*([A-Za-z]+)\s*\]',s)
            return "\n<input value='%s' name='%s' type='button'/>" % (m.groups(0)[0], m.groups(0)[1])
        
        return HtmlMode.modeEvalNode(self, node, depth, cDir, sideFx)

class PhpMode(HtmlMode) :

    def subPercent(self, s) :
        r = re.compile(r'\%([A-Za-z]+)')
        return r.sub(r'<? echo $\1 ?>',s)
        
    def modeEvalNode(self, node, depth = 0, cDir = '', sideFx = lambda x : x) :
        s = self.evalSymbols(node.text).strip()
        joinChildren = jFact(self.evalNode, depth, cDir, sideFx)
        
        self.log('PHPMode modeEvalNode : "%s"' % s)

        if s == '' : return ''
        
        if s[0] == '=' :
            var = s[1:].strip()
            return "<? $%s = $HTTP_GET_VARS['%s']; ?>" % (var,var)
        
        if re.match('\?:',s) :
            # cond in PHP
            build = []
            if len(node.children) == 0 :
                return ''
            i = 1
            build.append("""<? if %s {
%s
}""" % (node.children[0].text, node.children[0].children[0].text))
            if len(node.children) == 1 :
                build.append(' ?>')
                return ''.join(build)
            
            while len(node.children) - 1 > i :
                build.append(""" elseif %s {
%s
}""" % (node.children[i].text, self.evalNode(node.children[i].children[0], depth, cDir, sideFx)))
                i=i+1
            build.append(""" else {
%s
} ?>""" % (self.evalNode(node.children[i].children[0], depth, cDir, sideFx)))
            return ''.join(build)

        if re.match('"""',s) :
            return """echo '%s';""" % joinChildren(node)
        
        s = HtmlMode.modeEvalNode(self, node, depth, cDir, sideFx)
        return self.subPercent(s)


class JavascriptMode(UrMode) :

    """
    Right now, the Javascript mode does nothing but put some useful newlines.
    We may find decide on some other short-hands later
    """
    def modeEvalNode(self, node,  depth = 0, cDir = '', sideFx = lambda x : x) :
        s = self.evalSymbols(node.text)
        joinChildren = jFact(self.evalNode, depth, cDir, sideFx)
        b = []

        if len(s) > 0 :

            if '{;}'.find(s[-1]) >= 0 :
                b.append(s + '\n')
                
            else :
                b.append(s)

            b.append( joinChildren(node))
        else :
            # text value of this node is empty but children may still have value
            b.append(s)
            b.append( joinChildren(node))
            
        return ''.join(b)



