
import unittest
import string

from gwNode import *
from SymbolTable import *
from gwModes import *
from gwInterpreter import *

class TestMiscTools(unittest.TestCase) :

    def testSU(self) :
        self.assertEquals(spaceUnder('ab cd ef/xy z'),'ab_cd_ef/xy_z')

    
class TestSymbolTable(unittest.TestCase) :
    
    def testSymbolTable(self) :
        st = SymbolTable()
        self.assertEquals(st.depth(),1)

        st.pushFrame({"hello" : "teenage america",
                 "goodbye" : "cruel world"})
        self.assertEquals(st.get("hello"),"teenage america")
        self.assertEquals(st.get("goodbye"),"cruel world")

        self.assertEquals(st.depth(),2)
        st.pushFrame({"hello" : "coal black smith"})
        self.assertEquals(st.get("hello"),"coal black smith")
        self.assertEquals(st.get("goodbye"),"cruel world")
        self.assertEquals(st.get("hello"),"coal black smith")
        self.assertEquals(st.depth(),3)
        st.popFrame()
        self.assertEquals(st.get("hello"),"teenage america")
        self.assertEquals(st.get("goodbye"),"cruel world")
        self.assertEquals(st.depth(),2)

        for x in range(10) :
            st.pushFrame({'x':'%s'%x, 'y':'%s'%x*x})
        for y in range(9,0,-1) :
            self.assertEquals(st.get("hello"),"teenage america")
            self.assertEquals(st.get('x'),'%s'%y)
            st.popFrame()


class testLexers(unittest.TestCase) :

    def testLexLine(self) :
        ll = LexLine('.class','.')
        self.assertEquals(ll.matches,True)
        self.assertEquals(ll.cls,'class')
        self.assertEquals(ll.id,'')
        self.assertEquals(ll.args,[])
        ll = LexLine('.class','*')
        self.assertEquals(ll.matches,False)
        self.assertEquals(ll.cls,'')
        self.assertEquals(ll.id,'')
        ll = LexLine('.happy;fred','.')
        self.assertEquals(ll.matches,True)
        self.assertEquals(ll.cls,'happy')
        self.assertEquals(ll.id,'fred')
        
        ll = LexLine('.happy;fred 1,, 2,, 3','.')
        self.assertEquals(ll.matches,True)
        self.assertEquals(ll.cls,'happy')
        self.assertEquals(ll.id,'fred')
        self.assertEquals(ll.args,['1','2','3'])

        ll = LexLine('#@ csvName','#@')
        self.assertEquals(ll.matches,True)
        self.assertEquals(ll.key,'csvName')

        ll = LexLine('#@ csvName ;','#@')
        self.assertEquals(ll.matches,True)
        self.assertEquals(ll.args[0],';')

        self.assertEquals(ll.key,'csvName')


    def testLexPageName(self) :
        lpn = LexPageName('not a page')
        self.assertEquals(lpn.matches,False)
        
        lpn = LexPageName('&main')
        self.assertEquals(lpn.matches,True)
        self.assertEquals(lpn.pageName,'main')
        self.assertEquals(lpn.symbol,'')

        
        lpn = LexPageName('&main :page')
        self.assertEquals(lpn.matches,True)
        self.assertEquals(lpn.pageName,'main')
        self.assertEquals(lpn.outFileName,'main.html')
        self.assertEquals(lpn.symbol,'page')
        
        lpn = LexPageName('&main > zmain')
        self.assertEquals(lpn.matches,True)
        self.assertEquals(lpn.pageName,'main')
        self.assertEquals(lpn.outFileName,'zmain')
        
        lpn = LexPageName('&main    > main.xml')
        self.assertEquals(lpn.matches,True)
        self.assertEquals(lpn.pageName,'main')
        self.assertEquals(lpn.outFileName,'main.xml')

        lpn = LexPageName('&main  :svgDiag  > main.svg')
        self.assertEquals(lpn.matches,True)
        self.assertEquals(lpn.pageName,'main')
        self.assertEquals(lpn.outFileName,'main.svg')
        self.assertEquals(lpn.symbol,'svgDiag')


class TestGWNode(unittest.TestCase) :

    def testGWNode(self) :
        t = GWNode('hello')
        self.assertEquals(t.__class__,GWNode)
        t = GWNode('hello').addChild('world')
        self.assertEquals(t.children[0].__class__, GWNode)
        self.assertEquals(t.children[0].text, 'world')
        self.assertEquals(t.size(t),2)
        t = GWNode('hello').addChild(GWNode('teenage').addChild('america'))
        self.assertEquals(t.size(t),3)
        t.addChild('coal black smith')
        self.assertEquals(t.size(t),4)
        self.assertEquals(t.children[0].children[0].text,'america')

class TestArgBlock(unittest.TestCase) :

    def buildInterpreter(self) :
        i = Interpreter(SymbolTable(),SiteMapper(Logger()))
        return i

    def testArgBlock(self) :
        t = GWNode('__ a,, b,, c')
        t.addChild(GWNode('x : 1'))
        t.addChild(GWNode('y : 2'))
        t.addChild(GWNode('these'))
        t.addChild(GWNode('are'))
        t.addChild(GWNode('child'))
        t.addChild(GWNode('args'))

        t2=GWNode('z : ')
        t2.addChild(GWNode('has'))
        t2.addChild(GWNode(' more'))
        t.addChild(t2)

        t2=GWNode('something')
        t2.addChild(GWNode(' further'))
        t2.addChild(GWNode(' with more'))
        t.addChild(t2)

        i = self.buildInterpreter()
        m = i.modes['ur']
        ab = m.argBlock(t)
        self.assertEquals(ab.__class__,ArgBlock)
        self.assertEquals(len(ab.keys()),11)
        self.assertEquals(ab['_0'],'a')
        self.assertEquals(ab['_1'],'b')
        self.assertEquals(ab['_2'],'c')

        self.assertEquals(ab['x'],'1')
        self.assertEquals(ab['y'],'2')

        self.assertEquals(ab['__0'],'these')
        self.assertEquals(ab['__1'],'are')
        self.assertEquals(ab['__2'],'child')
        self.assertEquals(ab['__3'],'args')

        self.assertEquals(ab['z'],'has more')

        self.assertEquals(ab['__4'],'something further with more')
        
        t.text = ':blockCall a,, b,, c'
        ab = m.argBlock(t)
        self.assertEquals(len(ab.keys()),11)
        self.assertEquals(ab['_0'],'a')
        self.assertEquals(ab['x'],'1')
        self.assertEquals(ab['__4'],'something further with more')
        
    
class TestModes(unittest.TestCase) :

    def buildInterpreter(self) :
        i = Interpreter(SymbolTable(),SiteMapper(Logger()))
        return i

    def testEvalSymbol(self) :
        i = self.buildInterpreter()
        i.getSymbolTable().pushFrame({"x" : "world"})
        m = i.modes["ur"]
        self.assertEquals(m.evalSymbols("hello $x"),"hello world")
        self.assertEquals(m.evalSymbols("hello $/x/magazine/"),"hello world")
        self.assertEquals(m.evalSymbols("hello $/y/magazine/"),"hello magazine")
        
        self.assertEquals(m.evalSymbols("hello $x $x"),"hello world world")      
        self.assertEquals(m.evalSymbols("hello $/x/magazine/ $/x/magazine/"),"hello world world")
        self.assertEquals(m.evalSymbols("hello $/y/magazine/ $/y/magazine/"),"hello magazine magazine")
        
        self.assertEquals(m.evalSymbols("hello $/y/$x/"),"hello world")

        m = i.modes["staticSite"]
        self.assertEquals(m.evalSymbols("&$/y/$x/ wide web"),"&world wide web")

        self.assertEquals(m.evalSymbols(":$/x// a,, b,, c"),":world a,, b,, c")

    def testBasicEval(self) :
        i = self.buildInterpreter()

        t = GWNode('.myClass')
        self.assertEquals(i.evalNode(t),"<div class='myClass'></div>")

        t = GWNode('.myClass Some Stuff')
        self.assertEquals(i.evalNode(t),"<div class='myClass'>Some Stuff</div>")
        
        t = GWNode(';myId')
        self.assertEquals(i.evalNode(t),"<div id='myId'></div>")

        t = GWNode(';myId Some Stuff')
        self.assertEquals(i.evalNode(t),"<div id='myId'>Some Stuff</div>")

        t = GWNode('> NooRanch > http://www.nooranch.com')
        self.assertEquals(i.evalNode(t),"<a href='http://www.nooranch.com'>NooRanch</a>")

        i.setSiteRoot('http://www.geekweaver.com/')
        t = GWNode(' hello [[teenage/america.html world]] and more')
        self.assertEquals(i.evalNode(t)," hello <a href='http://www.geekweaver.com/teenage/america.html'>world</a> and more")
        
        t = GWNode(']myImg.gif')
        self.assertEquals(i.evalNode(t),"<img src='myImg.gif' alt=''/>")

        t = GWNode(']myImg.gif Alt Text')
        self.assertEquals(i.evalNode(t),"<img src='myImg.gif' alt='Alt Text'/>")

        t = GWNode('.<mytag>')
        t2 = GWNode('hello xml')
        t.addChild(t2)
        self.assertEquals(i.evalNode(t),"""<mytag>hello xml</mytag>""")

        t = GWNode('.<mytag atr="1">')
        t2 = GWNode('hello xml')
        t.addChild(t2)
        self.assertEquals(i.evalNode(t),"""<mytag atr="1">hello xml</mytag>""")

        t = GWNode('.<mytag a="1" b="2">')
        t2 = GWNode('hello xml')
        t.addChild(t2)
        self.assertEquals(i.evalNode(t),"""<mytag a="1" b="2">hello xml</mytag>""")


        t = GWNode('<mytag>')
        t2 = GWNode('hello xml')
        t.addChild(t2)
        self.assertEquals(i.evalNode(t),"""<mytag>hello xml""")
    
        t = GWNode('')
        self.assertEquals(i.evalNode(t),'')

        t = GWNode('// jfhj').addChild(GWNode('blah blah blah'))
        self.assertEquals(i.evalNode(t),'')        

    

    def testBlockEvals(self) :
        i = self.buildInterpreter()
        self.assertEquals(i.tree,None)
        self.assertEquals(i.getSymbolTable().__class__, SymbolTable)

        i.getSymbolTable().pushFrame({'x' : 'teenage', 'y' : 'america'})
        mode = i.modes['html']
        self.assertEquals(mode.evalSymbols('hello $x $y'),'hello teenage america')

        self.assertEquals(mode.evalSymbols('goodbye $z'),'goodbye $z')

        # simple block
        t = GWNode('::block').addChild(GWNode('goodbye $name'))

        i.getSymbolTable().pushFrame({'block' : t})

        t3 = GWNode('hello teenage america')
        self.assertEquals(mode.evalNode(t3),'hello teenage america')
        
        t4 = GWNode(':block')

        self.assertEquals(mode.evalNode(t4),'goodbye $name')
                                     
        t5 = GWNode('name : cruel world')
        t4.addChild(t5)

        self.assertEquals(mode.evalNode(t4),'goodbye cruel world')

        tx = GWNode(':block').addChild(GWNode('name :').addChild(GWNode('.<p>').addChild(GWNode('porkpie hat'))))

        self.assertEquals(i.evalNode(tx,'html'),'goodbye <p>porkpie hat</p>')

        i2 = self.buildInterpreter()
        t0 = GWNode('root')
        t0.addChild(t)
        t0.addChild(t3)
        t0.addChild(t4)
        mode = i2.modes['html']
                        
        self.assertEquals(mode.evalNode(t0),'roothello teenage americagoodbye cruel world')

        t6 = GWNode(':block')
        t7 = GWNode('name:')
        t8 = GWNode('.c')
        t9 = GWNode('w00t!')
        t6.addChild(t7)
        t7.addChild(t8)
        t8.addChild(t9)

        t0.addChild(t6)
        self.assertEquals(mode.evalNode(t0),"roothello teenage americagoodbye cruel worldgoodbye <div class='c'>w00t!</div>")
        

        t0 = GWNode('')
        t1 = GWNode('! "%s" % (5 * 5 * 5)')
        t0.addChild(t1)
        self.assertEquals(i2.evalNode(t0),'125')
        
        
        t = GWNode('*myList')
        self.assertEquals(i2.evalNode(t),"<ul class='myList'></ul>")

        t1 = GWNode("cat")
        t2 = GWNode("dog")
        t.addChild(t1)
        t.addChild(t2)
        self.assertEquals(i2.evalNode(t),"""<ul class='myList'>
<li>cat</li>
<li>dog</li></ul>""")

        t2.addChild(GWNode('wolf'))
        
                    

    def testTable(self) :
        i2 = self.buildInterpreter()
        
        t = GWNode("#myTable")
        self.assertEquals(i2.evalNode(t),"<table class='myTable'>\n</table>")
        r1 = GWNode("")
        r2 = GWNode("")
        r3 = GWNode("4,, 5,, 6")
        
        c11 = GWNode("oranges")
        c12 = GWNode("apples")
        c13 = GWNode("bananas")
        c21 = GWNode("1")
        c22 = GWNode("2")
        c23 = GWNode("3")

        t.addChild(r1)
        t.addChild(r2)
        r1.addChild(c11)
        r1.addChild(c12)
        r1.addChild(c13)
        r2.addChild(c21)
        r2.addChild(c22)
        r2.addChild(c23)
        self.assertEquals(i2.evalNode(t),"""<table class='myTable'>
<tr><td>oranges</td><td>apples</td><td>bananas</td></tr>
<tr><td>1</td><td>2</td><td>3</td></tr>
</table>""")

        t.addChild(r3)
        self.assertEquals(i2.evalNode(t),"""<table class='myTable'>
<tr><td>oranges</td><td>apples</td><td>bananas</td></tr>
<tr><td>1</td><td>2</td><td>3</td></tr>
<tr><td>4</td><td>5</td><td>6</td></tr>
</table>""")

        t = GWNode("#myTable;tid hello,, teenage,, america")
        t.addChild(r3)
        self.assertEquals(i2.evalNode(t),"""<table class='myTable' id='tid'>
<tr><th>hello</th><th>teenage</th><th>america</th></tr>
<tr><td>4</td><td>5</td><td>6</td></tr>
</table>""")
        

    def testForm(self) :
        i2 = self.buildInterpreter()
        t = GWNode("?myForm -> http://www.server.com/get")
        self.assertEquals(i2.evalNode(t),"<form name='myForm' method='GET' action='http://www.server.com/get'>\n</form>")
        t.addChild(GWNode("??hello"))
        self.assertEquals(i2.evalNode(t),"""<form name='myForm' method='GET' action='http://www.server.com/get'>
<input name='hello'/>
</form>""")
        t.addChild(GWNode("??Age:teenage"))
        self.assertEquals(i2.evalNode(t),"""<form name='myForm' method='GET' action='http://www.server.com/get'>
<input name='hello'/>
Age : <input name='teenage'/>
</form>""")
        t1 = GWNode('?*fruit')
        t1.addChild(GWNode('1:orange'))
        t1.addChild(GWNode('2:apple'))
        t1.addChild(GWNode('3:banana'))
        t.addChild(t1)
        self.assertEquals(i2.evalNode(t),"""<form name='myForm' method='GET' action='http://www.server.com/get'>
<input name='hello'/>
Age : <input name='teenage'/>
<select name='fruit'>
<option value='1'>orange</option>
<option value='2'>apple</option>
<option value='3'>banana</option>
</select>
</form>""")

        t.addChild(GWNode('[]'))
        self.assertEquals(i2.evalNode(t),"""<form name='myForm' method='GET' action='http://www.server.com/get'>
<input name='hello'/>
Age : <input name='teenage'/>
<select name='fruit'>
<option value='1'>orange</option>
<option value='2'>apple</option>
<option value='3'>banana</option>
</select>
<input type='submit'/>
</form>""")

        t.addChild(GWNode('[Go]'))
        self.assertEquals(i2.evalNode(t),"""<form name='myForm' method='GET' action='http://www.server.com/get'>
<input name='hello'/>
Age : <input name='teenage'/>
<select name='fruit'>
<option value='1'>orange</option>
<option value='2'>apple</option>
<option value='3'>banana</option>
</select>
<input type='submit'/>
<input value='Go' type='submit'/>
</form>""")

        t.addChild(GWNode('[Go ;goButton]'))
        self.assertEquals(i2.evalNode(t),"""<form name='myForm' method='GET' action='http://www.server.com/get'>
<input name='hello'/>
Age : <input name='teenage'/>
<select name='fruit'>
<option value='1'>orange</option>
<option value='2'>apple</option>
<option value='3'>banana</option>
</select>
<input type='submit'/>
<input value='Go' type='submit'/>
<input value='Go' name='goButton' type='button'/>
</form>""")

        t.text = "?myForm => http://www.server.com/post"
        ta = GWNode('?_ Description: desc ')
        t.addChild(ta)
        self.assertEquals(i2.evalNode(t),"""<form name='myForm' method='POST' action='http://www.server.com/post'>
<input name='hello'/>
Age : <input name='teenage'/>
<select name='fruit'>
<option value='1'>orange</option>
<option value='2'>apple</option>
<option value='3'>banana</option>
</select>
<input type='submit'/>
<input value='Go' type='submit'/>
<input value='Go' name='goButton' type='button'/>
Description : <textarea name='desc'>
</textarea>
</form>""")

        ta.text = '?_ desc '
        self.assertEquals(i2.evalNode(t),"""<form name='myForm' method='POST' action='http://www.server.com/post'>
<input name='hello'/>
Age : <input name='teenage'/>
<select name='fruit'>
<option value='1'>orange</option>
<option value='2'>apple</option>
<option value='3'>banana</option>
</select>
<input type='submit'/>
<input value='Go' type='submit'/>
<input value='Go' name='goButton' type='button'/>
<textarea name='desc'>
</textarea>
</form>""")

    def testArgsList(self) :
        i = self.buildInterpreter()
        t = GWNode('')
        t2 = GWNode('::argtest')
        t2.addChild( GWNode('xxx $_0 $_1 $_2'))
        t4 = GWNode(':argtest hello,, teenage,, america')
        t.addChild(t2)
        t.addChild(t4)
        i.evalNode(t)
        
        self.assertEquals(i.evalNode(t),"""xxx hello teenage america""")

    def testRep(self) :
        i = self.buildInterpreter()
        t = GWNode('xxx')
        t2 = GWNode('::rep')
        t2.addChild(GWNode('hello $name'))
        t3 = GWNode(':* rep')
        t4 = GWNode('')
        t4.addChild(GWNode('name : fred'))
        t5 = GWNode('')
        t5.addChild(GWNode('name : maria'))
        t6 = GWNode('')
        t6.addChild(GWNode('name : andrea'))
        t3.addChild(t4)
        t3.addChild(t5)
        t3.addChild(t6)
        t.addChild(t2)
        t.addChild(t3)
        self.assertEquals(i.evalNode(t),"""xxxhello fredhello mariahello andrea""")


    def testReps(self) :
        i = self.buildInterpreter()
        t = GWNode('')
        defs = GWNode('').addChild(GWNode('::f').addChild(GWNode('($_0),')))
        defs.addChild(GWNode('::sb').addChild(GWNode('[$_0],')))
        defs.addChild(GWNode('::g').addChild(GWNode(':forline f')))
        defs.addChild(GWNode('::h').addChild(GWNode(':forchild sb')))
        t.addChild(defs)
        t2 = GWNode(':g 1,, 2,, 3,, 4')
                    
        t.addChild(t2)
        self.assertEquals(i.evalNode(t),'(1),(2),(3),(4),')
        
        t2.text = ':h'
        t2.addChild(GWNode('xxx'))
        t2.addChild(GWNode('yyy'))
        t2.addChild(GWNode('key : val'))
        t2.addChild(GWNode('zzz').addChild('zzz'))

        self.assertEquals(i.evalNode(t),'[xxx],[yyy],[zzzzzz],')
        

    def testNamedTransitions(self) :
        st = SymbolTable()
        i = self.buildInterpreter()
        i.setSymbolTable(st)
        t = GWNode('blah')
        t2 = GWNode('&&htmlForm')
        t2.addChild(GWNode('[Form Mode]'))
        t.addChild(t2)
        i.evalNode(t)
        i.getLog().htmlFile('log.html')
        self.assertEquals(i.evalNode(t),"""blah\n<input value='Form Mode' type='submit'/>""")

    def testPHP(self) :
        i = self.buildInterpreter()
        t = GWNode('')
        t2 = GWNode('&&inter')
        t2.addChild(GWNode('=arg'))
        t.addChild(t2)
        i.evalNode(t)
        i.getLog().htmlFile('log.html')
        self.assertEquals(i.evalNode(t),"""<? $arg = $HTTP_GET_VARS['arg']; ?>""")        
        t3 = GWNode('.c')
        t3.addChild(GWNode('hello %arg and %more'))
        t2.addChild(t3)
        i.evalNode(t)
        i.getLog().htmlFile('log.html')
        self.assertEquals(i.evalNode(t),"""<? $arg = $HTTP_GET_VARS['arg']; ?><div class='c'>hello <? echo $arg ?> and <? echo $more ?></div>""")
        t = GWNode('')
        t2 = GWNode('&&inter')
        t.addChild(t2)
        t3 = GWNode('?:')
        t4 = GWNode("($arg < 0)")
        t4.addChild(GWNode("echo 'less';"))
        t5 = GWNode("($arg == 0)").addChild(GWNode('"""').addChild(GWNode('equals')))
        t6 = GWNode(" else ")
        t7 = GWNode('"""')
        t7.addChild(GWNode('more'))
        t6.addChild(t7)
        t2.addChild(t3)
        t3.addChild(t4)

        i.evalNode(t)
        i.getLog().htmlFile('log.html')
        self.assertEquals(i.evalNode(t),"""<? if ($arg < 0) {
echo 'less';
} ?>""")
        t3.addChild(t5)
        t3.addChild(t6)
        self.assertEquals(i.evalNode(t),"""<? if ($arg < 0) {
echo 'less';
} elseif ($arg == 0) {
echo 'equals';
} else {
echo 'more';
} ?>""")
        t = GWNode('').addChild(GWNode('&&inter').addChild(GWNode('"""').addChild('hello teenage america')))
        self.assertEquals(i.evalNode(t),"""echo 'hello teenage america';""")

    def testJavascript(self) :
        i = self.buildInterpreter()
        t = GWNode('&&javascript').addChild(GWNode('define f(x) {').addChild(GWNode(' return x*x;').addChild('}')))
        self.assertEquals(i.evalNode(t),"""define f(x) {
 return x*x;
}
""")

    
class TestInterpreter(unittest.TestCase) :

    def buildInterpreter(self) :
        i = Interpreter(SymbolTable(),SiteMapper(Logger()))
        return i



    def testRelativeRoot(self) :
        i = self.buildInterpreter()
        self.assertEquals(i.getRelativeRoot(4),'../../../../')

    def testForRefactoring(self) :
        st = SymbolTable()
        i = self.buildInterpreter()
        i.setSymbolTable(st)
        self.assertEquals(i.environment.symbolTable,st)
        self.assertEquals(i.modes['ur'].__class__,UrMode)
        self.assertEquals(i.modes['staticSite'].__class__,StaticSiteMode)
        self.assertEquals(i.modes['html'].__class__,HtmlMode)
        self.assertEquals(i.modes['htmlForm'].__class__,HtmlFormMode)
        self.assertEquals(i.modes['ur'].getMode('html').__class__,HtmlMode)
        self.assertEquals(i.modes['htmlForm'].getMode('staticSite').__class__,StaticSiteMode)
        self.assertEquals(i.modes['ur'].getMode('inter').__class__,PhpMode)
        self.assertEquals(i.modes['javascript'].__class__,JavascriptMode)
                                  
        


    def testSomeSimpleStuff(self) :
        t = GWNode('.<p>').addChild(GWNode('blah blah blah'))
        i = self.buildInterpreter()
        self.assertEquals(i.evalNode(t,'html'),'<p>blah blah blah</p>')
        t = GWNode('.<p>').addChild(t)
        self.assertEquals(i.evalNode(t,'html'),'<p><p>blah blah blah</p></p>')
        
        
    def testSiteMapper(self) :
        sm = SiteMapper(Logger())
        sm.add('ttest/site','hello.html','Hello','hello world')
        sm.add('ttest/site','goodbye.html','Goodbye','porkpie')
        sm.add('ttest/site/deeper','sm.html','sm','here will $siteMap be')
        sm.addExternalLink('ttest/site/deeper','http://www.synaesmedia.net','Synaesmedia')
        self.assertEquals(sm.getDir('ttest/site'),[['ttest/site','hello.html','Hello'],['ttest/site','goodbye.html','Goodbye']])
        self.assertEquals(sm.getULIndex('ttest/site'),"""
<ul>
<li><a href='hello.html'><img border='0' src='ttest/site/dTree/img/folder.gif'/>&nbsp;Hello</a></li>
<li><a href='goodbye.html'><img border='0' src='ttest/site/dTree/img/folder.gif'/>&nbsp;Goodbye</a></li>
</ul>""")                 
        sm.writeAll({'siteMap':'XYZ'})

    def testEvalSiteNodes(self) :
        i = self.buildInterpreter()
        st = i.getSymbolTable()
        self.assertEquals(st.__class__,SymbolTable)
        st.pushFrame({"name" : "phil"})
        self.assertEquals(i.modes['ur'].evalSymbols('hello world'),'hello world')
        self.assertEquals(i.modes['ur'].evalSymbols('hello $name'),'hello phil')
        self.assertEquals(i.modes['ur'].evalSymbols('hello ${name}zinho'),'hello philzinho')

        
if __name__ == '__main__' :
    unittest.main()
